@@ -254,7 +254,12 @@ impl Core {
         {
             Ok(false) => match msg {
                 SystemMsg::NodeCmd(_)
+                | SystemMsg::JoinResponse(_)
+                | SystemMsg::JoinRequest(_)
+                | SystemMsg::JoinAsRelocatedRequest(_)
+                | SystemMsg::BouncedUntrustedMessage { .. }
                 | SystemMsg::DkgStart { .. }
+                | SystemMsg::DkgFailureAgreement(_)
                 | SystemMsg::DkgMessage { .. }
                 | SystemMsg::DkgFailureObservation { .. }
                 | SystemMsg::NodeQuery(_)
@@ -265,6 +270,7 @@ impl Core {
                         msg_authority,
                         dst_location,
                         sender,
+                        known_keys,
                     };
 
                     Ok(vec![cmd])
@@ -275,7 +281,6 @@ impl Core {
                         msg_id,
                         msg,
                         msg_authority,
-                        known_keys,
                     };
 
                     Ok(vec![cmd])
@@ -300,7 +305,6 @@ impl Core {
         msg_id: MessageId,
         msg_authority: NodeMsgAuthority,
         node_msg: SystemMsg,
-        known_keys: Vec<BlsPublicKey>,
     ) -> Result<Vec<Command>> {
         let src_name = msg_authority.name();
 
@@ -387,42 +391,6 @@ impl Core {
 
                 Ok(vec![Command::TestConnectivity(name)])
             }
-            SystemMsg::JoinRequest(join_request) => {
-                trace!("Handling msg: JoinRequest from {}", sender);
-                self.handle_join_request(msg_authority.peer(sender)?, *join_request)
-                    .await
-            }
-            SystemMsg::JoinAsRelocatedRequest(join_request) => {
-                trace!("Handling msg: JoinAsRelocatedRequest from {}", sender);
-                if self.is_not_elder()
-                    && join_request.section_key == *self.section.chain().last_key()
-                {
-                    return Ok(vec![]);
-                }
-
-                self.handle_join_as_relocated_request(
-                    msg_authority.peer(sender)?,
-                    *join_request,
-                    known_keys,
-                )
-                .await
-            }
-            SystemMsg::BouncedUntrustedMessage {
-                msg: bounced_msg,
-                dst_section_pk,
-            } => {
-                trace!("Handling msg: BouncedUntrustedMessage from {}", sender);
-
-                Ok(self.handle_bounced_untrusted_message(
-                    msg_authority.peer(sender)?,
-                    dst_section_pk,
-                    *bounced_msg,
-                )?)
-            }
-            SystemMsg::DkgFailureAgreement(sig_set) => {
-                trace!("Handling msg: Dkg-FailureAgreement from {}", sender);
-                self.handle_dkg_failure_agreement(&src_name, &sig_set)
-            }
             SystemMsg::Propose {
                 ref content,
                 ref sig_share,
@@ -477,10 +445,6 @@ impl Core {
 
                 Ok(commands)
             }
-            SystemMsg::JoinResponse(join_response) => {
-                debug!("Ignoring unexpected message: {:?}", join_response);
-                Ok(vec![])
-            }
             SystemMsg::JoinAsRelocatedResponse(join_response) => {
                 trace!("Handling msg: JoinAsRelocatedResponse from {}", sender);
                 if let Some(RelocateState::InProgress(ref mut joining_as_relocated)) =
@@ -528,10 +492,54 @@ impl Core {
         dst_location: DstLocation,
         node_msg: SystemMsg,
         sender: SocketAddr,
+        known_keys: Vec<BlsPublicKey>,
     ) -> Result<Vec<Command>> {
         let src_name = msg_authority.name();
         trace!("Handling non blocking message");
         match node_msg {
+            SystemMsg::JoinResponse(join_response) => {
+                debug!(
+                    "Ignoring unexpected join response message: {:?}",
+                    join_response
+                );
+                Ok(vec![])
+            }
+            SystemMsg::DkgFailureAgreement(sig_set) => {
+                trace!("Handling msg: Dkg-FailureAgreement from {}", sender);
+                self.handle_dkg_failure_agreement(&src_name, &sig_set).await
+            }
+            SystemMsg::JoinRequest(join_request) => {
+                trace!("Handling msg: JoinRequest from {}", sender);
+                self.handle_join_request(msg_authority.peer(sender)?, *join_request)
+                    .await
+            }
+            SystemMsg::JoinAsRelocatedRequest(join_request) => {
+                trace!("Handling msg: JoinAsRelocatedRequest from {}", sender);
+                if self.is_not_elder()
+                    && join_request.section_key == *self.section.chain().last_key()
+                {
+                    return Ok(vec![]);
+                }
+
+                self.handle_join_as_relocated_request(
+                    msg_authority.peer(sender)?,
+                    *join_request,
+                    known_keys,
+                )
+                .await
+            }
+            SystemMsg::BouncedUntrustedMessage {
+                msg: bounced_msg,
+                dst_section_pk,
+            } => {
+                trace!("Handling msg: BouncedUntrustedMessage from {}", sender);
+
+                Ok(self.handle_bounced_untrusted_message(
+                    msg_authority.peer(sender)?,
+                    dst_section_pk,
+                    *bounced_msg,
+                )?)
+            }
             SystemMsg::DkgStart {
                 session_id,
                 elder_candidates,
@@ -718,6 +726,7 @@ impl Core {
             let aggregation = false;
 
             self.send_node_msg_to_targets(msg, target_holders, aggregation)
+                .await
         } else {
             error!("Received unexpected message while Adult");
             Ok(vec![])
@@ -725,7 +734,7 @@ impl Core {
     }
 
     /// Takes a message and forms commands to send to specified targets
-    pub(super) fn send_node_msg_to_targets(
+    pub(super) async fn send_node_msg_to_targets(
         &self,
         msg: SystemMsg,
         targets: BTreeSet<XorName>,
@@ -750,7 +759,7 @@ impl Core {
             let src = our_name;
 
             WireMsg::for_dst_accumulation(
-                self.key_share().map_err(|err| err)?,
+                &self.key_share().await.map_err(|err| err)?,
                 src,
                 dummy_dst_location,
                 msg,
